/*
    WinCdArb - Windows CdRom Device Arbiter service

    Copyright (C) 2007-2019 GuinpinSoft inc <wincdarb@makemkv.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#ifndef NTIPC_H_INCLUDED
#define NTIPC_H_INCLUDED

#define IPC_MESSAGE_TYPE_CONNECTION_REQUEST 10

typedef struct _IPC_MESSAGE {
    USHORT      DataLength;
    USHORT      Length;
    USHORT      MessageType;
    USHORT      DataInfoOffset;
    ULONG       UniqueProcessId;
    ULONG       UniqueProcessIdHigh64;
    // ...
} IPC_MESSAGE,*PIPC_MESSAGE;

#define IPC_HEADER_LENGTH_32    0x18
#define IPC_HEADER_LENGTH_64    0x28
#define IPC_MAX_HEADER_LENGTH   IPC_HEADER_LENGTH_64

#define IPC_MAX_DATA_LENGTH     0x130

#ifdef __cplusplus
extern "C" {
#endif

NTSTATUS NTAPI NtCreatePort(
    OUT PHANDLE             PortHandle,
    IN  POBJECT_ATTRIBUTES  ObjectAttributes,
    IN  ULONG               MaxConnectInfoLength,
    IN  ULONG               MaxDataLength,
    IN  PVOID               Reserved);

NTSTATUS NTAPI NtReplyWaitReceivePort(
    IN  HANDLE              PortHandle,
    OUT PVOID               Reserved1 OPTIONAL,
    IN  PVOID               Reserved2 OPTIONAL,
    OUT PIPC_MESSAGE        IncomingRequest);

NTSTATUS NTAPI NtReplyWaitReceivePortEx(
    IN  HANDLE              PortHandle,
    OUT PVOID               Reserved1 OPTIONAL,
    IN  PVOID               Reserved2 OPTIONAL,
    OUT PIPC_MESSAGE        IncomingRequest,
    IN  PLARGE_INTEGER      Timeout OPTIONAL);

NTSTATUS NTAPI NtConnectPort(
    OUT PHANDLE             ClientPortHandle,
    IN  PUNICODE_STRING     ServerPortName,
    IN  PSECURITY_QUALITY_OF_SERVICE SecurityQos,
    IN  PVOID               Reserved1  OPTIONAL,
    IN  PVOID               Reserved2  OPTIONAL,
    OUT PULONG              MaximumMessageLength OPTIONAL,
    IN  PUCHAR              ConnectionInfo OPTIONAL,
    IN  PULONG              ConnectionInfoLength OPTIONAL);

NTSTATUS NTAPI NtAcceptConnectPort(
    OUT PHANDLE             ServerPortHandle,
    IN  PVOID               Reserved1  OPTIONAL,
    IN  PIPC_MESSAGE        ConnectionReply,
    IN  BOOLEAN             AcceptConnection,
    IN  PVOID               Reserved2  OPTIONAL,
    IN  PVOID               Reserved3  OPTIONAL);

NTSTATUS NTAPI NtCompleteConnectPort(
    IN HANDLE               PortHandle);


#ifdef __cplusplus
}
#endif

#endif // NTIPC_H_INCLUDED

