/*
    WinCdArb - Windows CdRom Device Arbiter service

    Copyright (C) 2007-2019 GuinpinSoft inc <wincdarb@makemkv.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include <windows.h>
#include <winnt.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include "cdarbsrv.h"

static int Use(const wchar_t* argv0)
{
    static const char use_text[]=
        "\n"
        "  Please see the source code for all availbale commands.\n"
        "\n";

    const wchar_t* p = wcsrchr(argv0,'\\');
    if (NULL==p)
    {
        p = argv0;
    } else {
        p++;
    }

    fwprintf(stdout,L"Service name : %s\n",WinCdArbServiceDisplayName);
    fwprintf(stdout,L"Description  : %s\n",WinCdArbServiceDescription);
    fprintf(stdout,  "Version      : %s\n",WinCdArbVersionString());

    fputs("\n",stdout);

    fwprintf(stdout,L"Use:\n  %s <command> [arguments]\n",p);

    fputs(use_text,stdout);
    fputs(WinCdArbServiceLicense,stdout);

    fflush(stdout);

    return 1;
}

extern "C"
int __cdecl wmain(int argc,wchar_t **argv)
{
    if (argc<2)
    {
#ifndef WINCDARB_BUILD_INSTALLER_ONLY
        WinCdArbServiceControlMain();
#endif //!WINCDARB_BUILD_INSTALLER_ONLY
        return Use(argv[0]);
    }

    auto MainCommand =[argv](const wchar_t* Command,const char* ActionText,const char* SuccessText,const char* FailureText,HRESULT(*ActionProc)())
    {
        if (0==wcscmp(argv[1],Command))
        {
            HRESULT hr;

            fprintf(stdout,ActionText);
            __try
            {
                hr = (*ActionProc)();
            }
            __except (EXCEPTION_EXECUTE_HANDLER)
            {
                hr = GetExceptionCode();
            }
            fprintf(stdout,(S_OK==hr)?SuccessText:FailureText,hr);
            exit((int)hr);
        }
    };

#ifndef WINCDARB_BUILD_INSTALLER_ONLY // no service code

    MainCommand(L"testMain",
        "Runnung server as console test application\n",
        "Server exited\n",
        "Server exited with status=%08x\n",
        []()
    {
        WinCdArbServerInit();
        return WinCdArbServerLoop();
    });

    MainCommand(L"installSelf",
        "Installing service...\n",
        "Installed successfully\n",
        "Installation failed with status=%08x\n",
        []()
    {
        return WinCdArbInstall((LPCWSTR)(PVOID)InstallExePathCopyThis);
    });

    MainCommand(L"registerSelf",
        "Registering service...\n",
        "Registered successfully\n",
        "Registration failed with status=%08x\n",
        []()
    {
        return WinCdArbInstall((LPCWSTR)(PVOID)InstallExePathUseThis);
    });

    if (0==wcscmp(argv[1],L"test"))
    {
        HRESULT hr = WinCdArbClientTest(argc>3?argv[2]:NULL);

        fprintf(stdout,"Test complete hr=%08x\n",hr);
        return 0;
    }

    if (0==wcscmp(argv[1],L"version"))
    {
        fprintf(stdout,"%s\n",WinCdArbVersionString());
        return 0;
    }
#endif // !WINCDARB_BUILD_INSTALLER_ONLY

    MainCommand(L"installTiny",
        "Installing (tiny) service...\n",
        "Installed successfully\n",
        "Installation failed with status=%08x\n",
        []()
    {
        return WinCdArbInstall((LPCWSTR)(PVOID)InstallExePathCopyTiny);
    });

    MainCommand(L"start",
        "Starting service...\n",
        "Started successfully\n",
        "Start failed with status=%08x\n",
        []()
    {
        return WinCdArbServiceStart();
    });

    MainCommand(L"stop",
        "Stopping service...\n",
        "Stopped successfully\n",
        "Stop failed with status=%08x\n",
        []()
    {
        return WinCdArbServiceStop();
    });

    if ((argc>=3) && (0==wcscmp(argv[1],L"installExe")))
    {
        return WinCdArbInstall(argv[2]);
    }

    return 0;
}

