/*
    WinCdArb - Windows CdRom Device Arbiter service

    Copyright (C) 2007-2019 GuinpinSoft inc <wincdarb@makemkv.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include "winntapi.h"
#include "cdarbsrv.h"

const PCWSTR WinCdArbServiceDisplayName = L"CdRom Device Arbiter service";
const PCWSTR WinCdArbServiceDescription = L"Allows any application without Administrator privileges to use CdRom devices (GPL open-source license)";

static void WakeUpServerThread()
{
    ULONG len;
    UCHAR buffer[1];

    buffer[0]=0;
    len = 1;

    WinIpcClientRequest(WINCDARB_PORT_NAME,buffer,&len);
}

static DWORD WINAPI WinCdArbServiceCtrlHandler(DWORD dwOpcode,DWORD dwEvent,PVOID pEventData,PVOID pContext)
{
    DWORD   err;

    switch (dwOpcode)
    {
    case SERVICE_CONTROL_STOP:
    case SERVICE_CONTROL_SHUTDOWN:
        WinIpcSetExitServerLoopFlag();
        WakeUpServerThread();
        err = ERROR_SUCCESS;
        break;
    default:
        err = ERROR_CALL_NOT_IMPLEMENTED;
        break;
    }

    return err;
}

static VOID WINAPI WinCdArbServiceMain(DWORD /*argc*/,LPWSTR* /*argv*/)
{
    SERVICE_STATUS_HANDLE   hService;
    SERVICE_STATUS          svcStatus;

    hService = RegisterServiceCtrlHandlerExW(
        WINCDARB_SERVICE_NAME,
        WinCdArbServiceCtrlHandler,
        NULL);

    if (NULL==hService)
    {
        return;
    }

    WinCdArbServerInit();

    memset(&svcStatus,0,sizeof(svcStatus));
    svcStatus.dwServiceType         = SERVICE_WIN32_OWN_PROCESS;
    svcStatus.dwControlsAccepted    = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
    svcStatus.dwCurrentState        = SERVICE_RUNNING;

    if (FALSE==SetServiceStatus(hService,&svcStatus))
    {
        return;
    }

    // service body
    WinCdArbServerLoop();

    svcStatus.dwCurrentState        = SERVICE_STOPPED;
    if (FALSE==SetServiceStatus(hService,&svcStatus))
    {
        return;
    }

    return;
}

static const SERVICE_TABLE_ENTRYW gServiceTable[] = {
    { WINCDARB_SERVICE_NAME , WinCdArbServiceMain },
    { NULL , NULL }
};

void WINAPI WinCdArbServiceControlMain()
{
    if (FALSE==StartServiceCtrlDispatcherW(gServiceTable))
    {
        if (ERROR_FAILED_SERVICE_CONTROLLER_CONNECT==GetLastError())
        {
            // We were launched by the user, not SCM
            return;
        }
    }

    // We were launched by SCM, and we are done.
    ExitProcess(0);
}

